#include "PokktScene.h"
#include "../view/OfferwallView.h"
#include "../view/AdDemoView.h"
#include "../view/DemoSelectorView.h"
#include <stdlib.h>

USING_NS_CC;


CCScene* PokktScene::createScene()
{
    // 'scene' is an autorelease object
    CCScene* scene = CCScene::create();
    
    // 'layer' is an autorelease object
    CCLayer* layer = PokktScene::create();

    // add layer as a child to scene
    scene->addChild(layer);
    scene->setTag(143);

    // return the scene
    return scene;
}

void PokktScene::handleEnterForeground()
{
    _currentView->handleEnterForeground();
}

// on "init" you need to initialize your instance
bool PokktScene::init()
{
    if (!CCLayerColor::initWithColor(ccc4(0, 204, 153, 255)))
        return false;
    
    CCSize visibleSize = CCDirector::sharedDirector()->getVisibleSize();
    
    // listen for screen/view change events
    CCNotificationCenter::sharedNotificationCenter()->addObserver(this, callfuncO_selector(PokktScene::handleChangeScene), "changeView", NULL);
    
    // default view
    _currentView = static_cast<DemoSelectorView*>(DemoSelectorView::getView());
    this->addChild(_currentView);
    
    // add a close button
    CCMenuItem* closeButton = BaseView::createButton("", menu_selector(PokktScene::handleCloseCallback), this, "CloseNormal.png", "CloseNormal.png");
	closeButton->setPosition(CCPoint(visibleSize.width - closeButton->getContentSize().width / 2, closeButton->getContentSize().height / 2));
    CCMenu* menu = CCMenu::create(closeButton, NULL);
    menu->setPosition(CCPointZero);
    this->addChild(menu);
    
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    closeButton->setVisible(false);
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    closeButton->setVisible(true);
#endif
    
    // version
    initVersion(PCPokktManager::getSDKVersion().c_str());
    
    return true;
}

void PokktScene::initVersion(const char *version)
{
    CCSize visibleSize = CCDirector::sharedDirector()->getVisibleSize();
    
    CCLabelTTF* lblVersion = BaseView::createLabel(version, 20, ccBLACK);
    lblVersion->setPosition(CCPoint(visibleSize.width - lblVersion->getContentSize().width, visibleSize.height - lblVersion->getContentSize().height));
    this->addChild(lblVersion);
}

void PokktScene::handleChangeScene(CCObject* pSender)
{
    ChangeViewParams* params = static_cast<ChangeViewParams*>(pSender);
    
    // initiate view change
    if (_currentView != NULL)
    {
        this->removeChild(_currentView, false);
        _currentView = NULL;
    }
    
    switch (params->viewToChange)
    {
        case BaseView::DemoSelectorView:
            _currentView = static_cast<DemoSelectorView*>(DemoSelectorView::getView());
            break;
            
        case BaseView::OfferwallView:
            _currentView = static_cast<OfferwallView*>(OfferwallView::getView());
            break;
            
        case BaseView::AdDemoView:
            AdDemoView* view = static_cast<AdDemoView*>(AdDemoView::getView(params->adConfig->isRewarded));
            view->setAdConfig(params->adConfig);
            _currentView = view;
            break;
    }
    
    if (_currentView != NULL)
        this->addChild(_currentView);
}

void PokktScene::handleCloseCallback(CCObject* pSender)
{
#if (CC_TARGET_PLATFORM == CC_PLATFORM_WP8) || (CC_TARGET_PLATFORM == CC_PLATFORM_WINRT)
    MessageBox("You pressed the close button. Windows Store Apps do not implement a close button.","Alert");
    return;
#endif
    
    CCDirector::sharedDirector()->end();
    
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    exit(0);
#endif
}
