#include "BaseView.h"

using namespace cocos2d;


bool BaseView::init()
{
    if (!CCLayer::init())
        return false;
    
    visibleSize = CCDirector::sharedDirector()->getVisibleSize();
    
    // button container
    menuContainer = CCMenu::create();
    menuContainer->setPosition(CCPointZero);
    this->addChild(menuContainer, 1);
    
    // logo
    logo = CCSprite::create("logo.png");
    logo->setPosition(CCPoint(visibleSize.width / 2, visibleSize.height - logo->getContentSize().height/2));
    this->addChild(logo);
    
    backButton = createButton("", menu_selector(BaseView::handleBackPress), this, "back.png", "back.png");
    backButton->setScale(0.15f);
    backButton->setPosition(CCPoint(backButton->getContentSize().width / 2 * 0.15, backButton->getContentSize().height / 2 * 0.15));
    menuContainer->addChild(backButton);
    
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    backButton->setVisible(false);
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    backButton->setVisible(true);
#endif
    
    // for back press detection
    setKeypadEnabled(true);
    
    this->scheduleUpdate();
    
    return true;
}

BaseView::~BaseView()
{
    // to override
}

void BaseView::keyBackClicked()
{
    CCLayer::keyBackClicked();
    handleBackPress(NULL);
}

void BaseView::changeView(ChangeViewParams* params)
{
    CCNotificationCenter::sharedNotificationCenter()->postNotification("changeView", params);
}

void BaseView::changeLabelText(cocos2d::CCLabelTTF* label, const char* text)
{
    _labelsToUpdate[label] = text;
}

void BaseView::changeButtonLabel(cocos2d::CCMenuItem* button, const char* text)
{
    CCLabelTTF* label = static_cast<CCLabelTTF*>(button->getChildByTag(341)); // because I know!
    changeLabelText(label, text);
}

void BaseView::update(float dt)
{
    std::map<CCLabelTTF*, const char*>::iterator it = _labelsToUpdate.begin();
    for (; it != _labelsToUpdate.end(); ++it)
    {
        it->first->setString(it->second);
    }
    _labelsToUpdate.clear();
}

void BaseView::handleBackPress(CCObject*)
{
    // should override
}

void BaseView::handleEnterForeground()
{
    // should override
}

CCMenuItem* BaseView::createButton(const char *label,
                                   SEL_MenuHandler selector,
                                   CCObject *target,
                                   const char *pszNormalFileName,
                                   const char *pszDownFileName)
{
    CCMenuItemImage* button = CCMenuItemImage::create(pszNormalFileName, pszDownFileName, target, selector);
    CCLabelTTF *lblButtonText = createLabel(label, 28, cocos2d::ccWHITE);
    lblButtonText->setTag(341); // HACK: magic number
    lblButtonText->setPosition(CCPoint(button->getContentSize().width / 2, button->getContentSize().height / 2));
    button->addChild(lblButtonText);
    return button;
}

CCLabelTTF* BaseView::createLabel(const char *label, int fontSize, const cocos2d::ccColor3B &color, const cocos2d::CCPoint &position)
{
    CCLabelTTF *txtLabel = CCLabelTTF::create(label, "arial", fontSize);
    txtLabel->setColor(color);
    txtLabel->cocos2d::CCNode::setPosition(position);
    return txtLabel;
}