#include "OfferwallView.h"
#include "DemoSelectorView.h"


static OfferwallView* viewInstance = NULL;

bool OfferwallView::init()
{
    if (!BaseView::init())
        return false;
    
    _coinsEarned = 0;
    
    logo->setPosition(CCPoint(logo->getPosition().x, logo->getPosition().y + 100));
    
    int yPos = logo->getPosition().y - logo->getContentSize().height / 2;
    
    
    CCLabelTTF *lblPointEarned = BaseView::createLabel("Points Earned: ", 35, ccBLACK, CCPoint(visibleSize.width / 2 - 30, yPos));
    //lblPointEarned->setTextHorizontalAlignment(TextHAlignment::RIGHT);
    this->addChild(lblPointEarned);
    
    _txtPointsEarned = BaseView::createLabel("0", 35, ccBLACK,
                                             CCPoint(lblPointEarned->getContentSize().width / 2 + lblPointEarned->getPosition().x + 20, yPos));
    this->addChild(_txtPointsEarned);
    
    yPos -= 100;
    
    _btnGetOfferwallAny = createButton("Get Offerwall (Any Asset Value)", menu_selector(OfferwallView::handleGetOfferwallAny), this);
    _btnGetOfferwallAny->setPosition(CCPoint(visibleSize.width / 2, yPos));
    menuContainer->addChild(_btnGetOfferwallAny);
    
    yPos -= 100;
    
    _txiEnterMaxPoints = CCTextFieldTTF::textFieldWithPlaceHolder("Enter Max Points", "arial", 35);
    _txiEnterMaxPoints->setPosition(CCPoint(visibleSize.width / 2, yPos));
    //_txiEnterMaxPoints->setTextHorizontalAlignment(TextHAlignment::CENTER);
    this->addChild(_txiEnterMaxPoints);
    
    yPos -= 80;
    
    _btnGetOfferwallFix = createButton("Get Offerwall (Fix asset value)", menu_selector(OfferwallView::handleGetOfferwallFix), this, "btn_purple_normal.png", "btn_purple_down.png");
    _btnGetOfferwallFix->setPosition(CCPoint(visibleSize.width / 2, yPos));
    menuContainer->addChild(_btnGetOfferwallFix);
    
    yPos -= 100;
    
    _btnIncompleteCallback = createButton("Incomplete callback", menu_selector(OfferwallView::handleIncompleteCallback), this);
    _btnIncompleteCallback->setPosition(CCPoint(visibleSize.width / 2, yPos));
    menuContainer->addChild(_btnIncompleteCallback);
    
    yPos -= 120;
    
    _btnExportLog = createButton("Export Log", menu_selector(OfferwallView::handleExportLog), this);
    _btnExportLog->setPosition(CCPoint(visibleSize.width / 2, yPos));
    _btnExportLog->setScale(0.7f);
    menuContainer->addChild(_btnExportLog);
    
    yPos -= 100;
    
    // pokkt init for offerwall
    PCPokktManager::checkOfferwallCampaign();
    PCPokktManager::getPendingCoins();
    
    // listen for pokkt events
    PCPokktManager::setOfferwallEventListener(COIN_RESPONSE_EVENT, PC_OFFERWALL_EVENT_SELECTOR(OfferwallView::handleCoinResponse), this);
    PCPokktManager::setOfferwallEventListener(COIN_RESPONSE_WITH_TRANS_ID_EVENT, PC_OFFERWALL_EVENT_SELECTOR(OfferwallView::handleCoinResponseWithTrId), this);
    PCPokktManager::setOfferwallEventListener(COIN_RESPONSE_FAILED_EVENT, PC_OFFERWALL_EVENT_SELECTOR(OfferwallView::handleCoinResponseFailed), this);
    PCPokktManager::setOfferwallEventListener(CAMPAIGN_AVAILABILITY_EVENT, PC_OFFERWALL_EVENT_SELECTOR(OfferwallView::handleCampaignAvailibity), this);
    PCPokktManager::setOfferwallEventListener(OFFERWALL_CLOSED_EVENT, PC_OFFERWALL_EVENT_SELECTOR(OfferwallView::handleOfferwallclosed), this);
    
    return true;
}

OfferwallView::~OfferwallView()
{
    PCPokktManager::removeOfferwallEventListener(COIN_RESPONSE_EVENT);
    PCPokktManager::removeOfferwallEventListener(COIN_RESPONSE_WITH_TRANS_ID_EVENT);
    PCPokktManager::removeOfferwallEventListener(COIN_RESPONSE_FAILED_EVENT);
    PCPokktManager::removeOfferwallEventListener(CAMPAIGN_AVAILABILITY_EVENT);
    PCPokktManager::removeOfferwallEventListener(OFFERWALL_CLOSED_EVENT);
}

OfferwallView* OfferwallView::getView()
{
    if (viewInstance == NULL)
    {
        viewInstance = OfferwallView::create();
        viewInstance->retain();
    }
    return viewInstance;
}

void OfferwallView::handleBackPress(CCObject*)
{
    ChangeViewParams* params = new ChangeViewParams();
    params->viewToChange = DemoSelectorView;
    params->adConfig = NULL;
    changeView(params);
}

void OfferwallView::handleEnterForeground()
{
    PCPokktManager::getPendingCoins();
}

void OfferwallView::handleGetOfferwallAny(CCObject *pSender)
{
    PCPokktConfig* config = DemoSelectorView::getPokktConfig();
    config->offerwallAssetValue = "";
    PCPokktManager::getCoins(config);
}

void OfferwallView::handleGetOfferwallFix(CCObject *pSender)
{
    PCPokktConfig* config = DemoSelectorView::getPokktConfig();
    config->offerwallAssetValue = _txiEnterMaxPoints->getString();
    PCPokktManager::getCoins(config);
}

void OfferwallView::handleIncompleteCallback(CCObject *pSender)
{
    PCPokktManager::getPendingCoins();
}

void OfferwallView::handleExportLog(CCObject *pSender)
{
    PCPokktManager::exportLog();
}

void OfferwallView::handleCoinResponse(std::string coins)
{
    CCLOG("[POKKT-CPP] Coins earned from offerwall: %s", coins.c_str());
    
    int coinsInt = 0;
    std::stringstream(coins) >> coinsInt;
    _coinsEarned += coinsInt;
    
    char buf[100];
    sprintf(buf, "%d", _coinsEarned);
    _txtPointsEarned->setString(buf);
    
    // if auto-close is enables, close this screen
    //if (_chkCloseOnSuccess->isSelected())
    //    handleBackPress();
}

void OfferwallView::handleCoinResponseWithTrId(std::string coinsWithTrId)
{
    // extract comma separated values
    std::stringstream stream(coinsWithTrId);
    std::string value;
    std::vector<std::string> values;
    while (getline(stream, value, ','))
    {
        values.push_back(value);
        if (stream.peek() == ',')
            stream.ignore();
    }
    
    if (values.size() < 2)
        return; // the values received are not proper
    
    std::string points = values[0];
    std::string transId = values[1];
    
    if (points == "-1")
    {
        // no points earned
    }
    else
    {
        // points earned equal to coins with transaction id
    }

    
    if (values.size() >= 2)
        CCLOG("[POKKT-CPP] Coins earned with trId: %s, coins: %s.", values[1].c_str(), values[0].c_str());
    else
        CCLOG("[POKKT-CPP] Error receiving coins with transaction id!");
    
    _txtPointsEarned->setString(coinsWithTrId.c_str());
    
    // if auto-close is enables, close this screen
    //if (_chkCloseOnSuccess->isSelected())
    //    handleBackPress(NULL);
}

void OfferwallView::handleCoinResponseFailed(std::string message)
{
    // pending coins request fails
    CCLOG("[POKKT-CPP] Failed to get Pending coins!");
}

void OfferwallView::handleCampaignAvailibity(std::string message)
{
    bool available = message == "true";
    
    CCLOG("[POKKT-CPP] Offerwall Campaign availability: %s", (available ? "available" : "no campaigns"));
}

void OfferwallView::handleOfferwallclosed(std::string message)
{
    CCLOG("[POKKT-CPP] Offerwall Closed!");
}
