#include "PCPokktManager.h"
#include "PCParser.h"
#include <string>
#include <sstream>

#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)

#include "../androidExt/PCAndroidExtension.h"

#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)

#include "IOSExtension.h"

#endif

using namespace pokkt;


EVT_SELECTOR_POKKT_INITIALIZED PCPokktManager::_pokktInitializedListner = NULL;
cocos2d::CCObject* PCPokktManager::_pokktInitializedEventTarget = NULL;
std::map<std::string, PCPokktManager::ListenerTargetPairOfferwall*> PCPokktManager::_eventMapOfferwall;
std::map<std::string, PCPokktManager::ListenerTargetPairAdEvent*> PCPokktManager::_eventMapAdEvent;
std::map<std::string, PCPokktManager::ListenerTargetPairBannerEvent*> PCPokktManager::_eventMapBannerEvent;

// to_string() helper
template<typename T>
std::string to_string(const T& t)
{
    std::ostringstream os;
    os << t;
    return os.str();
}

void PCPokktManager::handleNativeMessage(std::string operation, std::string param)
{
    POKKTLOG("[POKKT-CPP] message received! operation: %s, param: %s", operation.c_str(), param.c_str());
    
    // pokkt initialized event
    if (operation.compare(POKKT_INITIALISED_EVENT) == 0)
    {
        json::Object jsonObject;
        std::stringstream strStream;
        strStream << param;
        json::Reader::Read(jsonObject, strStream);
        
        json::String& tempStrValue = jsonObject["isReady"];
        bool isReady = tempStrValue.Value().compare("true") == 0;
        tempStrValue = jsonObject["message"];
        std::string message = tempStrValue.Value();
        
        (_pokktInitializedEventTarget->*_pokktInitializedListner)(isReady, message);
        return;
    }
    
    // filter offerwall events
    if ((operation.compare(COIN_RESPONSE_EVENT) == 0) ||
        (operation.compare(COIN_RESPONSE_WITH_TRANS_ID_EVENT) == 0) ||
        (operation.compare(COIN_RESPONSE_FAILED_EVENT) == 0) ||
        (operation.compare(CAMPAIGN_AVAILABILITY_EVENT) == 0) ||
        (operation.compare(OFFERWALL_CLOSED_EVENT) == 0))
    {
        ListenerTargetPairOfferwall *l = _eventMapOfferwall[operation];
        if (l != NULL)
        {
            EVT_SELECTOR_OFFERWALL listener = l->listener;
            cocos2d::CCObject *target = l->target;
            (target->*listener)(param);
        }
    }
    
    // filter ad-events
    if ((operation.compare(AD_CACHING_COMPLETED_EVENT) == 0) ||
        (operation.compare(AD_CACHING_FAILED_EVENT) == 0) ||
        (operation.compare(AD_CLOSED_EVENT) == 0) ||
        (operation.compare(AD_COMPLETED_EVENT) == 0) ||
        (operation.compare(AD_DISPLAYED_EVENT) == 0) ||
        (operation.compare(AD_SKIPPED_EVENT) == 0) ||
        (operation.compare(AD_GRATIFIED_EVENT) == 0) ||
        (operation.compare(AD_AVAILABILITY_EVENT) == 0))
    {
        ListenerTargetPairAdEvent* l = _eventMapAdEvent[operation];
        if (l != NULL)
        {
            EVT_SELECTOR_AD_EVENT listener = l->listener;
            cocos2d::CCObject *target = l->target;
            
            // extract params
            std::map<std::string, std::string> extraValues;
            PCAdConfig* adConfig = fromAdConfigJsonString(param, extraValues);
            
            (target->*listener)(adConfig, extraValues);
        }
        return;
    }
    
    if (operation.compare(BANNER_LOADED) == 0)
    {
        ListenerTargetPairBannerEvent* l = _eventMapBannerEvent[operation];
        
        if (l != NULL)
        {
            std::string screenName = param;
            
            EVT_SELECTOR_BANNER listener = l->listener;
            cocos2d::CCObject *target = l->target;
            
            (target->*listener)(screenName, "");
        }
    }
    
    if (operation.compare(BANNER_LOAD_FAILED) == 0)
    {
        ListenerTargetPairBannerEvent* l = _eventMapBannerEvent[operation];
        
        if (l != NULL)
        {
            json::Object jsonObject;
            std::stringstream strStream;
            strStream << param;
            json::Reader::Read(jsonObject, strStream);
            
            json::String& tempStrValue = jsonObject["screenName"];
            std::string screenName = tempStrValue.Value();
            
            tempStrValue = jsonObject["errorMessage"];
            std::string errorMessage = tempStrValue.Value();
            
            EVT_SELECTOR_BANNER listener = l->listener;
            cocos2d::CCObject *target = l->target;
            
            (target->*listener)(screenName, errorMessage);
        }
        return;
    }
}

void PCPokktManager::setPokktInitializedListener(EVT_SELECTOR_POKKT_INITIALIZED listener, cocos2d::CCObject *target)
{
    _pokktInitializedListner = listener;
    _pokktInitializedEventTarget = target;
}

void PCPokktManager::removePokktInitializedListener()
{
    _pokktInitializedEventTarget = NULL;
    _pokktInitializedListner = NULL;
}

void PCPokktManager::setOfferwallEventListener(std::string eventName, EVT_SELECTOR_OFFERWALL listener, cocos2d::CCObject *target)
{
    ListenerTargetPairOfferwall* pair = new ListenerTargetPairOfferwall();
    pair->listener = listener;
    pair->target = target;
    _eventMapOfferwall[eventName] = pair;
}

void PCPokktManager::removeOfferwallEventListener(std::string eventName)
{
    CC_SAFE_DELETE(_eventMapOfferwall[eventName]);
    _eventMapOfferwall[eventName] = NULL;
}

void PCPokktManager::setAdEventListener(std::string eventName, EVT_SELECTOR_AD_EVENT listener, cocos2d::CCObject *target)
{
    ListenerTargetPairAdEvent* pair = new ListenerTargetPairAdEvent();
    pair->listener = listener;
    pair->target = target;
    _eventMapAdEvent[eventName] = pair;
}

void PCPokktManager::removeAdEventListener(std::string eventName)
{
    CC_SAFE_DELETE(_eventMapAdEvent[eventName]);
    _eventMapAdEvent[eventName] = NULL;
}

void PCPokktManager::setBannerEventListener(std::string eventName, EVT_SELECTOR_BANNER listener, cocos2d::CCObject *target)
{
    ListenerTargetPairBannerEvent* pair = new ListenerTargetPairBannerEvent();
    pair->listener = listener;
    pair->target = target;
    _eventMapBannerEvent[eventName] = pair;
}

void PCPokktManager::removeBannerEventListener(std::string eventName)
{
    CC_SAFE_DELETE(_eventMapBannerEvent[eventName]);
    _eventMapBannerEvent[eventName] = NULL;
}


void PCPokktManager::notifyNative(std::string operation, std::string param)
{
    POKKTLOG("[POKKT-CPP] notifying native of operation: %s, param: %s", operation.c_str(), param.c_str());
    
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    
    notifyAndroid(operation, param);
    
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    
    notifyIOS(operation.c_str(), param.c_str());
    
#endif
}

double PCPokktManager::getRewardedAdVcOnNative(PCAdConfig* adConfig)
{
    POKKTLOG("[POKKT-CPP] getting video-vc on native...");
    std::string param = toAdConfigJSONString(adConfig);
    
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    
    return getRewardedAdVcOnAndroid(param);
    
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    
    return getRewardedAdVcOnIOS(param.c_str());
    
#endif
}

std::string PCPokktManager::getSDKVersionOnNative()
{
    POKKTLOG("[POKKT-CPP] getting sdk version on native...");
    
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    
    return getSDKVersionOnAndroid();
    
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    
    return getSDKVersionOnIOS();
    
#endif
}

/**
 * PCParser
 **/
std::string toAdConfigJSONString(PCAdConfig* adConfig)
{
    json::Object jsonObject;
    
    jsonObject["screenName"] = json::String(adConfig->screenName);
    jsonObject["adFormat"] = json::String(to_string(adConfig->adFormat));
    jsonObject["isRewarded"] = json::String(adConfig->isRewarded ? "true" : "false");
    jsonObject["shouldAllowSkip"] = json::String(adConfig->shouldAllowSkip ? "true" : "false");
    jsonObject["defaultSkipTime"] = json::String(to_string(adConfig->defaultSkipTime));
    jsonObject["skipConfirmMessage"] = json::String(adConfig->skipConfirmMessage);
    jsonObject["backButtonDisabled"] = json::String(adConfig->backButtonDisabled ? "true" : "false");
    jsonObject["shouldAllowMute"] = json::String(adConfig->shouldAllowMute ? "true" : "false");
    jsonObject["shouldSkipConfirm"] = json::String(adConfig->shouldConfirmSkip ? "true" : "false");
    jsonObject["skipConfirmYesLabel"] = json::String(adConfig->skipConfirmYesLabel);
    jsonObject["skipConfirmNoLabel"] = json::String(adConfig->skipConfirmNoLabel);
    jsonObject["skipTimerMessage"] = json::String(adConfig->skipTimerMessage);
    jsonObject["incentiveMessage"] = json::String(adConfig-> incentiveMessage);
    
    std::stringstream strStream;
    json::Writer::Write(jsonObject, strStream);
    
    return strStream.str();
}

std::string toBannerParamsJSONString(std::string screenName, int bannerPosition)
{
    json::Object jsonObject;
    
    jsonObject["screenName"] = json::String(screenName);
    jsonObject["bannerPosition"] = json::String(to_string(bannerPosition));
    
    std::stringstream strStream;
    json::Writer::Write(jsonObject, strStream);
    
    return strStream.str();
}

std::string toBannerRectParamsJSONString(std::string screenName, int width, int height, int x, int y)
{
    json::Object jsonObject;
    
    jsonObject["screenName"] = json::String(screenName);
    jsonObject["width"] = json::String(to_string(width));
    jsonObject["height"] = json::String(to_string(height));
    jsonObject["x"] = json::String(to_string(x));
    jsonObject["y"] = json::String(to_string(y));

    std::stringstream strStream;
    json::Writer::Write(jsonObject, strStream);
    
    return strStream.str();
}


PCAdConfig* fromAdConfigJsonString(std::string param, std::map<std::string, std::string>& extraValues)
{
    PCAdConfig* adConfig = new PCAdConfig();
    
    try
    {
        json::Object jsonObject;
        std::stringstream strStream;
        strStream << param;
        json::Reader::Read(jsonObject, strStream);
        
        json::String& tempStrValue = jsonObject["screenName"];
        adConfig->screenName = tempStrValue.Value();
        
        tempStrValue = jsonObject["isRewarded"];
        adConfig->isRewarded = tempStrValue.Value().compare("true") == 0;
        
        tempStrValue = jsonObject["adFormat"];
        std::stringstream(tempStrValue.Value()) >> adConfig->adFormat;
        
        tempStrValue = jsonObject["backButtonDisabled"];
        adConfig->backButtonDisabled = tempStrValue.Value().compare("true") == 0;
        
        tempStrValue = jsonObject["defaultSkipTime"];
        std::stringstream(tempStrValue.Value()) >> adConfig->defaultSkipTime;
        
        tempStrValue = jsonObject["shouldAllowSkip"];
        adConfig->shouldAllowSkip = tempStrValue.Value().compare("true") == 0;
        
        tempStrValue = jsonObject["shouldAllowMute"];
        adConfig->shouldAllowMute = tempStrValue.Value().compare("true") == 0;
        
        tempStrValue = jsonObject["shouldSkipConfirm"];
        adConfig->shouldConfirmSkip = tempStrValue.Value().compare("true") == 0;
        
        tempStrValue = jsonObject["skipConfirmMessage"];
        adConfig->skipConfirmMessage = tempStrValue.Value();
        
        tempStrValue = jsonObject["skipConfirmYesLabel"];
        adConfig->skipConfirmYesLabel = tempStrValue.Value();
        
        tempStrValue = jsonObject["skipConfirmNoLabel"];
        adConfig->skipConfirmNoLabel = tempStrValue.Value();
        
        tempStrValue = jsonObject["skipTimerMessage"];
        adConfig->skipTimerMessage = tempStrValue.Value();
        
        tempStrValue = jsonObject["incentiveMessage"];
        adConfig->incentiveMessage = tempStrValue.Value();
        
        // fetch extraValues
        json::Object extraValuesJson = jsonObject["extraValues"];
        json::Object::const_iterator itValuesIterator(extraValuesJson.Begin());
        for (; itValuesIterator != extraValuesJson.End(); ++itValuesIterator) {
            const json::Object::Member& member = *itValuesIterator;
            tempStrValue = member.element;
            const std::string& name = member.name;
            const std::string& element = tempStrValue.Value();
            extraValues[name] = element;
        }
    }
    catch (std::runtime_error& e)
    {
        PCPokktManager::showLog("Parser.FromAdConfigJsonString :: failed to parse, exception: %s");
        PCPokktManager::showLog(e.what());
    }
    
    return adConfig;
}

std::string toIAPDetailsJSONString(IAPDetails* details)
{
    json::Object jsonObject;
    
    jsonObject["productId"] = json::String(details->productId);
    jsonObject["price"] = json::String(to_string(details->price));
    jsonObject["currencyCode"] = json::String(details->currencyCode);
    jsonObject["title"] = json::String(details->title);
    jsonObject["description"] = json::String(details->description);
    jsonObject["purchaseData"] = json::String(details->purchaseData);
    jsonObject["purchaseSignature"] = json::String(details->purchaseSignature);
    jsonObject["purchaseStore"] = json::String(details->purchaseStore);
    
    std::stringstream strStream;
    json::Writer::Write(jsonObject, strStream);
    
    return strStream.str();
}

std::string toPokktConfigJSONString(PCPokktConfig* config)
{
    json::Object jsonObject;
    
    jsonObject["applicationId"] = json::String(config->applicationId);
    jsonObject["securityKey"] = json::String(config->securityKey);
    jsonObject["integrationType"] = json::String(config->integrationType);
    jsonObject["thirdPartyUserId"] = json::String(config->thirdPartyUserId);
    
    jsonObject["offerwallAssetValue"] = json::String(config->offerwallAssetValue);
    jsonObject["closeOnSuccessFlag"] = json::String(config->closeOnSuccessFlag ? "true" : "false");
    
    jsonObject["name"] = json::String(config->name);
    jsonObject["age"] = json::String(config->age);
    jsonObject["sex"] = json::String(config->sex);
    jsonObject["mobileNo"] = json::String(config->mobileNo);
    jsonObject["emailAddress"] = json::String(config->emailAddress);
    jsonObject["location"] = json::String(config->location);
    jsonObject["birthday"] = json::String(config->birthday);
    jsonObject["maritalStatus"] = json::String(config->maritalStatus);
    jsonObject["facebookId"] = json::String(config->facebookId);
    jsonObject["twitterHandle"] = json::String(config->twitterHandle);
    jsonObject["education"] = json::String(config->education);
    jsonObject["nationality"] = json::String(config->nationality);
    jsonObject["employment"] = json::String(config->employment);
    jsonObject["maturityRating"] = json::String(config->maturityRating);
    
    jsonObject["selectedAnalyticsType"] = json::String(config->selectedAnalyticsType);
    jsonObject["googleAnalyticsID"] = json::String(config->googleAnalyticsID);
    jsonObject["mixPanelProjectToken"] = json::String(config->mixPanelProjectToken);
    jsonObject["flurryApplicationKey"] = json::String(config->flurryApplicationKey);
    
    std::stringstream strStream;
    json::Writer::Write(jsonObject, strStream);
    
    return strStream.str();
}
