#import <Foundation/Foundation.h>
#import "PokktSDK/PokktManager.h"
#import "PokktSDK/AdDelegate.h"
#import "PokktSDK/PokktConfig.h"
#import "PokktSDK/AdConfig.h"
#import "IOSExtension.h"
#import "PCPokktManager.h"

UIView* bannerContainer = nil;

@interface PokktAdDelegate:NSObject<AdDelegate, PokktInitDelegate, BannerDelegate>
@end

#define TOP_LEFT  1
#define TOP_CENTER  2
#define TOP_RIGHT  3
#define MIDDLE_LEFT  4
#define MIDDLE_CENTER  5
#define MIDDLE_RIGHT  6
#define BOTTOM_LEFT  7
#define BOTTOM_CENTER  8
#define BOTTOM_RIGHT  9

// util method definitions
void topLeft(UIView * view);
void topRight(UIView * view);
void topCenter(UIView * view);
void middleLeft(UIView* view);
void middleRight(UIView * view);
void middleCenter(UIView * view);
void bottomLeft(UIView * view);
void bottomRight(UIView * view);
void bottomCenter(UIView * view);
/***
 * end banner utils
 ***/


UIViewController* getRootViewController()
{
    UIViewController *viewController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
    return viewController;
}

NSDictionary* extractJSON(NSString* jsonString)
{
    NSError* jsonError;
    NSData* jsonData = [jsonString dataUsingEncoding:NSUTF8StringEncoding];
    NSDictionary* jsonObject = [NSJSONSerialization JSONObjectWithData:jsonData
                                                               options:NSJSONReadingMutableContainers
                                                                 error:&jsonError];
    return jsonObject;
}

PokktConfig* parsePokktConfig(NSString* configString)
{
    NSDictionary* jsonObject = extractJSON(configString);
    
    PokktConfig* config = [[PokktConfig alloc] init];
    
    // essential
    config.securityKey = jsonObject[@"securityKey"];
    config.applicationId = jsonObject[@"applicationId"];
    config.thirdPartyUserId = jsonObject[@"thirdPartyUserId"];
    
    // optional
    config.name = jsonObject[@"name"];
    config.age = jsonObject[@"age"];
    config.sex = jsonObject[@"sex"];
    config.mobileNumber = jsonObject[@"mobileNo"];
    config.emailAddress = jsonObject[@"emailAddress"];
    config.location = jsonObject[@"location"];
    config.birthday = jsonObject[@"birthday"];
    config.maritalStatus = jsonObject[@"maritalStatus"];
    config.facebookId = jsonObject[@"facebookId"];
    config.twitterHandle = jsonObject[@"twitterHandle"];
    config.educationInformation = jsonObject[@"education"];
    config.nationality = jsonObject[@"nationality"];
    config.employmentStatus = jsonObject[@"employment"];
    config.maturityRating = jsonObject[@"maturityRating"];
    
    // analytics
    config.googleTrackerID = jsonObject[@"googleAnalyticsID"];
    config.mixPanelTrackerID = jsonObject[@"mixPanelProjectToken"];
    config.flurryTrackerID = jsonObject[@"flurryApplicationKey"];
    
    if(jsonObject[@"selectedAnalyticsType"] != nil && ![jsonObject[@"selectedAnalyticsType"] isEqual:@""])
    {
        if ([jsonObject[@"selectedAnalyticsType"] isEqual:@"FLURRY"])
            config.eventType = FLURRY_ANALYTICS;
        
        else if ([jsonObject[@"selectedAnalyticsType"] isEqual:@"GOOGLE_ANALYTICS"])
            config.eventType = GOOGLE_ANALYTICS;
        
        else if ([jsonObject[@"selectedAnalyticsType"] isEqual:@"MIXPANNEL"])
            config.eventType = MIXPANNEL_ANALYTICS;
    }
    
    return config;
}

AdConfig* parseAdConfig(NSString* configString)
{
    NSDictionary* jsonObject = extractJSON(configString);
    
    AdConfig* adConfig = [[AdConfig alloc] init];
    
    adConfig.screenName = jsonObject[@"screenName"];
    adConfig.isRewarded = [jsonObject[@"isRewarded"] boolValue];
    adConfig.shouldAllowSkip = [jsonObject[@"shouldAllowSkip"] boolValue];
    adConfig.defaultSkipTime = [jsonObject[@"defaultSkipTime"] floatValue];
    adConfig.skipConfirmMessage = jsonObject[@"skipConfirmMessage"];
    adConfig.shouldAllowMute = [jsonObject[@"shouldAllowMute"] boolValue];
    adConfig.shouldConfirmSkip = [jsonObject[@"shouldSkipConfirm"] boolValue];
    adConfig.skipConfirmYesLabel = jsonObject[@"skipConfirmYesLabel"];
    adConfig.skipConfirmNoLabel = jsonObject[@"skipConfirmNoLabel"];
    adConfig.skipTimerMessage = jsonObject[@"skipTimerMessage"];
    adConfig.incentiveMessage = jsonObject[@"incentiveMessage"];
    adConfig.adFormat = [jsonObject[@"adFormat"] intValue];
    
    return adConfig;
}

NSString* stringifyAdConfig(AdConfig* adConfig, NSDictionary* extraValues)
{
    NSMutableDictionary* jsonObject = [[NSMutableDictionary alloc] init];
    
    jsonObject[@"screenName"] = adConfig.screenName;
    jsonObject[@"isRewarded"] = adConfig.isRewarded ? @"true" : @"false";
    jsonObject[@"shouldAllowSkip"] = adConfig.shouldAllowSkip ? @"true" : @"false";
    jsonObject[@"defaultSkipTime"] = [NSNumber numberWithInt:adConfig.defaultSkipTime];
    jsonObject[@"skipConfirmMessage"] = adConfig.skipConfirmMessage;
    jsonObject[@"shouldAllowMute"] = adConfig.shouldAllowMute ? @"true" : @"false";
    jsonObject[@"shouldSkipConfirm"] = adConfig.shouldConfirmSkip ? @"true" : @"false";
    jsonObject[@"skipConfirmYesLabel"] = adConfig.skipConfirmYesLabel;
    jsonObject[@"skipConfirmNoLabel"] = adConfig.skipConfirmNoLabel;
    jsonObject[@"skipTimerMessage"] = adConfig.skipTimerMessage;
    jsonObject[@"incentiveMessage"] = adConfig.incentiveMessage;
    jsonObject[@"adFormat"] = [@(adConfig.adFormat) stringValue];
    
    if (extraValues != NULL)
        jsonObject[@"extraValues"] = extraValues;
    
    NSError* jsonError;
    NSData* jsonData = [NSJSONSerialization dataWithJSONObject:jsonObject options:0 error:&jsonError];
    NSString* jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    return jsonString;
}

InAppPurchaseDetails* parseIAPDetails(NSString* detailsString)
{
    NSDictionary* jsonObject = extractJSON(detailsString);
    
    InAppPurchaseDetails* detail = [[InAppPurchaseDetails alloc] init];
    
    detail.productIdentifier = jsonObject[@"productId"];
    detail.productPrice = jsonObject[@"price"];
    detail.productCurrency = jsonObject[@"currencyCode"];
    detail.productTitle = jsonObject[@"title"];
    detail.productDescription = jsonObject[@"description"];
    
    // TODO
    //detail.setPurchaseData(jsonObject.optString("purchaseData"));
    //detail.setPurchaseSignature(jsonObject.optString("purchaseSignature"));
    //detail.setPurchaseStore(IAPStoreType.valueOf(jsonObject.optString("purchaseStore")));
    
    return detail;
}

void loadBanner(NSString* params)
{
    NSDictionary* jsonObject = extractJSON(params);
    
    NSString *screenName = jsonObject[@"screenName"];
    int position = [jsonObject[@"bannerPosition"] intValue];
    
    if(bannerContainer != NULL)
    {
        [PokktManager destroyBanner:bannerContainer];
        bannerContainer = NULL;
    }
    
    bannerContainer = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 320, 50)];
    bannerContainer.translatesAutoresizingMaskIntoConstraints = NO;
    [getRootViewController().view addSubview:bannerContainer];
    
    switch (position) {
        case 1: topLeft(bannerContainer);       break;
        case 2: topCenter(bannerContainer);     break;
        case 3: topRight(bannerContainer);      break;
        case 4: middleLeft(bannerContainer);    break;
        case 5: middleCenter(bannerContainer);  break;
        case 6: middleRight(bannerContainer);   break;
        case 7: bottomLeft(bannerContainer);    break;
        case 8: bottomCenter(bannerContainer);  break;
        case 9: bottomRight(bannerContainer);   break;
        default: topCenter(bannerContainer);    break;
    }
    
    [PokktManager loadBanner:screenName bannerView:bannerContainer withViewController:getRootViewController()];
}

void loadBannerWithRect(NSString *params)
{
    NSDictionary* jsonObject = extractJSON(params);
    
    NSString *screenName = jsonObject[@"screenName"];
    int width = [jsonObject[@"width"] intValue];
    int height = [jsonObject[@"height"] intValue];
    float x = [jsonObject[@"x"] floatValue];
    float y = [jsonObject[@"y"]floatValue];
    
    if(bannerContainer != NULL)
    {
        [PokktManager destroyBanner:bannerContainer];
        bannerContainer = NULL;
    }
    
    bannerContainer = [[UIView alloc] initWithFrame:CGRectMake(x, y, width, height)];
    [getRootViewController().view addSubview:bannerContainer];
    [PokktManager loadBanner:screenName bannerView:bannerContainer withViewController:getRootViewController()];
}

void notifyIOS(const char* operation, const char* param)
{
    NSString *operationNSStr = [NSString stringWithUTF8String:(operation)];
    NSString *paramNSStr = [NSString stringWithUTF8String:(param)];
    
    // Pokkt SDK Api
    if ([operationNSStr isEqualToString:@"setDebug"])
    {
        BOOL debug = [paramNSStr isEqualToString:@"true"];
        [PokktManager setDebug:debug];
    }
    
    else if ([operationNSStr isEqualToString:@"exportLog"])
        [PokktManager exportLog:getRootViewController()];
    
    else if ([operationNSStr isEqualToString:@"showToast"])
        [PokktManager showToast:paramNSStr viewController:getRootViewController()];
    
    else if ([operationNSStr isEqualToString:@"initPokkt"])
    {
        NSLog(@"InitPokkt PokktConfig: %@", paramNSStr);
        PokktAdDelegate *delegate = [[PokktAdDelegate alloc] init];
        [PokktManager setInitDelegate:delegate];
        [PokktManager setAdDelegate:delegate];
        [PokktManager setBannerDelegate:delegate];
        [PokktManager initPokkt:parsePokktConfig(paramNSStr)];
    }
    
    else if ([operationNSStr isEqualToString:@"updatePokktConfig"])
        [PokktManager updatePokktConfig:parsePokktConfig(paramNSStr)];
    
    // Session Methods
    else if ([operationNSStr isEqualToString:@"startSession"])
        [PokktManager startSession];
    
    else if ([operationNSStr isEqualToString:@"endSession"])
        [PokktManager endSession];
    
    else if ([operationNSStr isEqualToString:@"notifyAppInstall"])
        [PokktManager notifyAppInstall];
    
    else if ([operationNSStr isEqualToString:@"trackIAP"])
        [PokktManager trackIAP:parseIAPDetails(paramNSStr)];
    
    
    // OfferWall Methods -- NOT Supported in iOS Plugin  -- should go to default log case.
    
    
    // Ad Methods
    else if ([operationNSStr isEqualToString:@"cacheAd"])
        [PokktManager cacheAd:parseAdConfig(paramNSStr)];
    
    else if ([operationNSStr isEqualToString:@"showAd"])
        [PokktManager showAd:parseAdConfig(paramNSStr) viewController:getRootViewController()];
    
    else if ([operationNSStr isEqualToString:@"checkAdAvailability"])
        [PokktManager checkAdAvailability:parseAdConfig(paramNSStr)];
    
    // Banner Ad
    else if ([operationNSStr isEqualToString:@"loadBanner"])
        loadBanner(paramNSStr);
    
    else if ([operationNSStr isEqualToString:@"loadBannerRect"])
        loadBannerWithRect(paramNSStr);
    
    else if ([operationNSStr isEqualToString:@"bannerAutoRefresh"])
        [PokktManager setBannerAutoRefresh:[paramNSStr boolValue]];
    
    else if ([operationNSStr isEqualToString:@"removeBanner"])
    {
        if(bannerContainer != NULL)
            [PokktManager destroyBanner:bannerContainer];
    }
    
    else
        NSLog(@"[POKKT-NATIVE] Unknown operation requested: %@", operationNSStr);
}

float getRewardedAdVcOnIOS(const char* param)
{
    NSString *paramNSStr = [NSString stringWithUTF8String:(param)];
    return [PokktManager getRewardedAdVc:parseAdConfig(paramNSStr)];
}

const char* getSDKVersionOnIOS()
{
    const char* version = [[NSString stringWithFormat:@"%@", [PokktManager getSDKVersion]] UTF8String];
    return version;
}

/***
 * util method declarations
 ***/
NSLayoutConstraint* getLayoutConstraint(UIView* view, NSLayoutAttribute layoutAttribute1, NSLayoutRelation layoutRelation, UIView* toItem, NSLayoutAttribute layoutAttribute2, CGFloat multiplier, CGFloat constant)
{
    return [NSLayoutConstraint constraintWithItem:view
                                        attribute:layoutAttribute1
                                        relatedBy:layoutRelation
                                           toItem:toItem
                                        attribute:layoutAttribute2
                                       multiplier:multiplier
                                         constant:constant];
}

void updateBannerLocation(UIView* view, NSLayoutConstraint* leading, NSLayoutConstraint* bottom, NSLayoutConstraint* trailing, NSLayoutConstraint* height)
{
    [view addConstraint:trailing];
    [view.superview addConstraint:bottom];
    [view.superview addConstraint:leading];
    [view addConstraint:height];
}

//Top
void topLeft(UIView* view)
{
    NSLayoutConstraint* leading = getLayoutConstraint(view, NSLayoutAttributeLeft, NSLayoutRelationEqual, view.superview, NSLayoutAttributeLeft, 1.0f, 0.f);
    NSLayoutConstraint* bottom = getLayoutConstraint(view, NSLayoutAttributeTop, NSLayoutRelationEqual, view.superview, NSLayoutAttributeTop, 1.0f, 0.f);
    NSLayoutConstraint* trailing = getLayoutConstraint(view, NSLayoutAttributeWidth, NSLayoutRelationEqual, nil, NSLayoutAttributeWidth, 1.0f, 320);
    NSLayoutConstraint* height = getLayoutConstraint(view, NSLayoutAttributeHeight, NSLayoutRelationEqual, nil, NSLayoutAttributeNotAnAttribute, 0, 50);
    
    updateBannerLocation(view, leading, bottom, trailing, height);
}

void topRight(UIView * view)
{
    NSLayoutConstraint* leading = getLayoutConstraint(view, NSLayoutAttributeRight, NSLayoutRelationEqual, view.superview, NSLayoutAttributeRight, 1.0f, 0.f);
    NSLayoutConstraint* bottom = getLayoutConstraint(view, NSLayoutAttributeTop, NSLayoutRelationEqual, view.superview, NSLayoutAttributeTop, 1.0f, 0.f);
    NSLayoutConstraint* trailing = getLayoutConstraint(view, NSLayoutAttributeWidth, NSLayoutRelationEqual, nil, NSLayoutAttributeWidth, 1.0f, 320);
    NSLayoutConstraint* height = getLayoutConstraint(view, NSLayoutAttributeHeight, NSLayoutRelationEqual, nil, NSLayoutAttributeNotAnAttribute, 0, 50);
    
    updateBannerLocation(view, leading, bottom, trailing, height);
}

void topCenter(UIView * view)
{
    NSLayoutConstraint* leading = getLayoutConstraint(view, NSLayoutAttributeCenterX, NSLayoutRelationEqual, view.superview, NSLayoutAttributeCenterX, 1.0f, 0.f);
    NSLayoutConstraint* bottom = getLayoutConstraint(view, NSLayoutAttributeTop, NSLayoutRelationEqual, view.superview, NSLayoutAttributeTop, 1.0f, 0.f);
    NSLayoutConstraint* trailing = getLayoutConstraint(view, NSLayoutAttributeWidth, NSLayoutRelationEqual, nil, NSLayoutAttributeWidth, 1.0f, 320);
    NSLayoutConstraint* height = getLayoutConstraint(view, NSLayoutAttributeHeight, NSLayoutRelationEqual, nil, NSLayoutAttributeNotAnAttribute, 0, 50);
    
    updateBannerLocation(view, leading, bottom, trailing, height);
}

// Middle
void middleLeft(UIView* view)
{
    NSLayoutConstraint* leading = getLayoutConstraint(view, NSLayoutAttributeLeft, NSLayoutRelationEqual, view.superview, NSLayoutAttributeLeft, 1.0f, 0.f);
    NSLayoutConstraint* bottom = getLayoutConstraint(view, NSLayoutAttributeCenterY, NSLayoutRelationEqual, view.superview, NSLayoutAttributeCenterY, 1.0f, 0.f);
    NSLayoutConstraint* trailing = getLayoutConstraint(view, NSLayoutAttributeWidth, NSLayoutRelationEqual, nil, NSLayoutAttributeWidth, 1.0f, 320);
    NSLayoutConstraint* height = getLayoutConstraint(view, NSLayoutAttributeHeight, NSLayoutRelationEqual, nil, NSLayoutAttributeNotAnAttribute, 0, 50);
    
    updateBannerLocation(view, leading, bottom, trailing, height);
}

void middleRight(UIView * view)
{
    NSLayoutConstraint* leading = getLayoutConstraint(view, NSLayoutAttributeRight, NSLayoutRelationEqual, view.superview, NSLayoutAttributeRight, 1.0f, 0.f);
    NSLayoutConstraint* bottom = getLayoutConstraint(view, NSLayoutAttributeCenterY, NSLayoutRelationEqual, view.superview, NSLayoutAttributeCenterY, 1.0f, 0.f);
    NSLayoutConstraint* trailing = getLayoutConstraint(view, NSLayoutAttributeWidth, NSLayoutRelationEqual, nil, NSLayoutAttributeWidth, 1.0f, 320);
    NSLayoutConstraint* height = getLayoutConstraint(view, NSLayoutAttributeHeight, NSLayoutRelationEqual, nil, NSLayoutAttributeNotAnAttribute, 0, 50);
    
    updateBannerLocation(view, leading, bottom, trailing, height);
}

void middleCenter(UIView * view)
{
    NSLayoutConstraint* leading = getLayoutConstraint(view, NSLayoutAttributeCenterX, NSLayoutRelationEqual, view.superview, NSLayoutAttributeCenterX, 1.0f, 0.f);
    NSLayoutConstraint* bottom = getLayoutConstraint(view, NSLayoutAttributeCenterY, NSLayoutRelationEqual, view.superview, NSLayoutAttributeCenterY, 1.0f, 0.f);
    NSLayoutConstraint* trailing = getLayoutConstraint(view, NSLayoutAttributeWidth, NSLayoutRelationEqual, nil, NSLayoutAttributeWidth, 1.0f, 320);
    NSLayoutConstraint* height = getLayoutConstraint(view, NSLayoutAttributeHeight, NSLayoutRelationEqual, nil, NSLayoutAttributeNotAnAttribute, 0, 50);
    
    updateBannerLocation(view, leading, bottom, trailing, height);
}

//Bottom
void bottomLeft(UIView * view)
{
    NSLayoutConstraint* leading = getLayoutConstraint(view, NSLayoutAttributeLeading, NSLayoutRelationEqual, view.superview, NSLayoutAttributeLeading, 1.0f, 0.f);
    NSLayoutConstraint* bottom = getLayoutConstraint(view, NSLayoutAttributeBottom, NSLayoutRelationEqual, view.superview, NSLayoutAttributeBottom, 1.0f, 0.f);
    NSLayoutConstraint* trailing = getLayoutConstraint(view, NSLayoutAttributeWidth, NSLayoutRelationEqual, nil, NSLayoutAttributeWidth, 1.0f, 320);
    NSLayoutConstraint* height = getLayoutConstraint(view, NSLayoutAttributeHeight, NSLayoutRelationEqual, nil, NSLayoutAttributeNotAnAttribute, 0, 50);
    
    updateBannerLocation(view, leading, bottom, trailing, height);
}

void bottomRight(UIView * view)
{
    NSLayoutConstraint* leading = getLayoutConstraint(view, NSLayoutAttributeRight, NSLayoutRelationEqual, view.superview, NSLayoutAttributeRight, 1.0f, 0.f);
    NSLayoutConstraint* bottom = getLayoutConstraint(view, NSLayoutAttributeBottom, NSLayoutRelationEqual, view.superview, NSLayoutAttributeBottom, 1.0f, 0.f);
    NSLayoutConstraint* trailing = getLayoutConstraint(view, NSLayoutAttributeWidth, NSLayoutRelationEqual, nil, NSLayoutAttributeWidth, 1.0f, 320);
    NSLayoutConstraint* height = getLayoutConstraint(view, NSLayoutAttributeHeight, NSLayoutRelationEqual, nil, NSLayoutAttributeNotAnAttribute, 0, 50);
    
    updateBannerLocation(view, leading, bottom, trailing, height);
}

void bottomCenter(UIView * view)
{
    NSLayoutConstraint* leading = getLayoutConstraint(view, NSLayoutAttributeCenterX, NSLayoutRelationEqual, view.superview, NSLayoutAttributeCenterX, 1.0f, 0.f);
    NSLayoutConstraint* bottom = getLayoutConstraint(view, NSLayoutAttributeBottom, NSLayoutRelationEqual, view.superview, NSLayoutAttributeBottom, 1.0f, 0.f);
    NSLayoutConstraint* trailing = getLayoutConstraint(view, NSLayoutAttributeWidth, NSLayoutRelationEqual, nil, NSLayoutAttributeWidth, 1.0f, 320);
    NSLayoutConstraint* height = getLayoutConstraint(view, NSLayoutAttributeHeight, NSLayoutRelationEqual, nil, NSLayoutAttributeNotAnAttribute, 0, 50);
    
    updateBannerLocation(view, leading, bottom, trailing, height);
}

// events (already defined)
#define POKKT_INITIALISED_EVENT         "PokktInitialised"
#define AD_CACHING_COMPLETED_EVENT      "AdCachingCompleted"
#define AD_CACHING_FAILED_EVENT         "AdCachingFailed"
#define AD_CLOSED_EVENT                 "AdClosed"
#define AD_COMPLETED_EVENT              "AdCompleted"
#define AD_DISPLAYED_EVENT              "AdDisplayed"
#define AD_SKIPPED_EVENT                "AdSkipped"
#define AD_GRATIFIED_EVENT              "AdGratified"
#define AD_AVAILABILITY_EVENT           "AdAvailability"
#define BANNER_LOADED                   "BannerLoaded"
#define BANNER_LOAD_FAILED              "BannerLoadFailed"


@implementation PokktAdDelegate

// use this to notify the framework observer
- (void) notifyFramework:(NSString*) operation param:(NSString*) param
{
    pokkt::PCPokktManager::handleNativeMessage([operation UTF8String], [param UTF8String]);
}

// Pokkt Init Related Callback
- (void)onPokktInitialised: (BOOL)isReady withMessage:(NSString *)message
{
    NSMutableDictionary* jsonObject = [[NSMutableDictionary alloc] init];
    
    jsonObject[@"isReady"] = (isReady ? @"true" : @"false");
    jsonObject[@"message"] = message;
    NSError* jsonError;
    NSData* jsonData = [NSJSONSerialization dataWithJSONObject:jsonObject options:0 error:&jsonError];
    NSString* jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    
    [self notifyFramework:@POKKT_INITIALISED_EVENT param:jsonString];
}

// Video/Interstitial Ad Related Callback
- (void)onAdCachingCompleted:(AdConfig *)adConfig reward:(float)reward
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"AD_VC"] = [[NSNumber numberWithFloat:reward] stringValue];
    [self notifyFramework:@AD_CACHING_COMPLETED_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

- (void)onAdCachingFailed:(AdConfig *)adConfig errorMessage:(NSString *)errorMessage
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"MESSAGE"] = errorMessage;
    [self notifyFramework:@AD_CACHING_FAILED_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

- (void)onAdCompleted:(AdConfig *)adConfig
{
    [self notifyFramework:@AD_COMPLETED_EVENT param:stringifyAdConfig(adConfig, NULL)];
}

- (void)onAdDisplayed:(AdConfig *)adConfig
{
    [self notifyFramework:@AD_DISPLAYED_EVENT param:stringifyAdConfig(adConfig, NULL)];
}

- (void)onAdGratified:(AdConfig *)adConfig reward:(float)reward
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"REWARD"] = [[NSNumber numberWithFloat:reward] stringValue];
    [self notifyFramework:@AD_GRATIFIED_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

- (void)onAdSkipped:(AdConfig *)adConfig
{
    [self notifyFramework:@AD_SKIPPED_EVENT param:stringifyAdConfig(adConfig, NULL)];
}

- (void)onAdClosed:(AdConfig *)adConfig
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"BACK_BUTTON"] = @"false";
    [self notifyFramework:@AD_CLOSED_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

- (void)onAdAvailabilityStatus:(AdConfig *)adConfig isAdAvailable:(BOOL)isAdAvailable
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"IS_AVAILABLE"] = (isAdAvailable ? @"true" : @"false");
    [self notifyFramework:@AD_AVAILABILITY_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

// Banner Related Callback
- (void)onBannerLoaded:(NSString *)screenName
{
    [self notifyFramework:@BANNER_LOADED param:screenName];
}

- (void)onBannerLoadFailed:(NSString *)screenName errorMessage:(NSString *)errorMessage
{
    NSMutableDictionary* jsonObject = [[NSMutableDictionary alloc] init];
    
    jsonObject[@"screenName"] = screenName;
    jsonObject[@"errorMessage"] = errorMessage;
    NSError* jsonError;
    NSData* jsonData = [NSJSONSerialization dataWithJSONObject:jsonObject options:0 error:&jsonError];
    NSString* jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    
    [self notifyFramework:@BANNER_LOAD_FAILED param:jsonString];
}


@end
